@@ -32,7 +32,7 @@
 void LoadGraph::clear_background()
 {
 	if (this->background) {
-		g_object_unref(this->background);
+		cairo_surface_destroy (this->background);
 		this->background = NULL;
 	}
 }
@@ -67,6 +67,7 @@ unsigned LoadGraph::num_bars() const
 
 #define FRAME_WIDTH 4
 void draw_background(LoadGraph *g) {
+	GtkAllocation allocation;
 	double dash[2] = { 1.0, 2.0 };
 	cairo_t *cr;
 	guint i;
@@ -80,12 +81,13 @@ void draw_background(LoadGraph *g) {
 	g->graph_delx = (g->draw_width - 2.0 - g->rmargin - g->indent) / (LoadGraph::NUM_POINTS - 3);
 	g->graph_buffer_offset = (int) (1.5 * g->graph_delx) + FRAME_WIDTH ;
 
-	g->background = gdk_pixmap_new (GDK_DRAWABLE (g->disp->window),
-					g->disp->allocation.width,
-					g->disp->allocation.height,
-					-1);
-	cr = gdk_cairo_create (g->background);
-	
+	gtk_widget_get_allocation (g->disp, &allocation);
+        g->background = gdk_window_create_similar_surface (gtk_widget_get_window (g->disp),
+                                                           CAIRO_CONTENT_COLOR,
+                                                           allocation.width,
+                                                           allocation.height);
+	cr = cairo_create (g->background);
+
 	// set the background colour
 	GtkStyle *style = gtk_widget_get_style (ProcData::get_instance()->notebook);
 	gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_NORMAL]);
@@ -93,13 +95,13 @@ void draw_background(LoadGraph *g) {
 
 	/* draw frame */
 	cairo_translate (cr, FRAME_WIDTH, FRAME_WIDTH);
-	
+
 	/* Draw background rectangle */
 	cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 	cairo_rectangle (cr, g->rmargin + g->indent, 0,
 			 g->draw_width - g->rmargin - g->indent, g->real_draw_height);
 	cairo_fill(cr);
-	
+
 	cairo_set_line_width (cr, 1.0);
 	cairo_set_dash (cr, dash, 2, 0);
 	cairo_set_font_size (cr, g->fontsize);
@@ -167,7 +169,7 @@ void draw_background(LoadGraph *g) {
 
 /* Redraws the backing buffer for the load graph and updates the window */
 void
-load_graph_draw (LoadGraph *g)
+load_graph_queue_draw (LoadGraph *g)
 {
 	/* repaint */
 	gtk_widget_queue_draw (g->disp);
@@ -180,40 +182,41 @@ load_graph_configure (GtkWidget *widget,
 		      GdkEventConfigure *event,
 		      gpointer data_ptr)
 {
+	GtkAllocation allocation;
 	LoadGraph * const g = static_cast<LoadGraph*>(data_ptr);
-	g->draw_width = widget->allocation.width - 2 * FRAME_WIDTH;
-	g->draw_height = widget->allocation.height - 2 * FRAME_WIDTH;
+
+	gtk_widget_get_allocation (widget, &allocation);
+	g->draw_width = allocation.width - 2 * FRAME_WIDTH;
+	g->draw_height = allocation.height - 2 * FRAME_WIDTH;
 
 	g->clear_background();
 
-	if (g->gc == NULL) {
-		g->gc = gdk_gc_new (GDK_DRAWABLE (widget->window));
-	}
-
-	load_graph_draw (g);
+	load_graph_queue_draw (g);
 
 	return TRUE;
 }
 
 static gboolean
-load_graph_expose (GtkWidget *widget,
-		   GdkEventExpose *event,
-		   gpointer data_ptr)
+load_graph_draw (GtkWidget *widget,
+                 cairo_t * context,
+                 gpointer data_ptr)
 {
 	LoadGraph * const g = static_cast<LoadGraph*>(data_ptr);
+	GtkAllocation allocation;
+	GdkWindow *window;
 
 	guint i, j;
 	gdouble sample_width, x_offset;
 
+	window = gtk_widget_get_window (g->disp);
+	gtk_widget_get_allocation (g->disp, &allocation);
+
 	if (g->background == NULL) {
 		draw_background(g);
-	}
-	gdk_draw_drawable (g->disp->window,
-			   g->gc,
-			   g->background,
-			   0, 0, 0, 0, 
-			   g->disp->allocation.width, 
-			   g->disp->allocation.height);
+                cairo_pattern_t * pattern = cairo_pattern_create_for_surface (g->background);
+		gdk_window_set_background_pattern (window, pattern);
+                cairo_pattern_destroy (pattern);
+        }
 
 	/* Number of pixels wide for one graph point */
 	sample_width = (float)(g->draw_width - g->rmargin - g->indent) / (float)LoadGraph::NUM_POINTS;
@@ -226,7 +229,7 @@ load_graph_expose (GtkWidget *widget,
 	/* draw the graph */
 	cairo_t* cr;
 
-	cr = gdk_cairo_create (g->disp->window);
+	cr = gdk_cairo_create (window);
 
 	cairo_set_line_width (cr, 1);
 	cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
@@ -242,7 +245,7 @@ load_graph_expose (GtkWidget *widget,
 		for (i = 1; i < LoadGraph::NUM_POINTS; ++i) {
 			if (g->data[i][j] == -1.0f)
 				continue;
-			cairo_curve_to (cr, 
+			cairo_curve_to (cr,
 				       x_offset - ((i - 0.5f) * g->graph_delx),
 				       (1.0f - g->data[i-1][j]) * g->real_draw_height + 3.5f,
 				       x_offset - ((i - 0.5f) * g->graph_delx),
@@ -365,16 +368,16 @@ get_memory (LoadGraph *g)
 }
 
 static void
-net_scale (LoadGraph *g, unsigned din, unsigned dout)
+net_scale (LoadGraph *g, guint64 din, guint64 dout)
 {
 	g->data[0][0] = 1.0f * din / g->net.max;
 	g->data[0][1] = 1.0f * dout / g->net.max;
 
-	unsigned dmax = std::max(din, dout);
+        guint64 dmax = std::max(din, dout);
 	g->net.values[g->net.cur] = dmax;
 	g->net.cur = (g->net.cur + 1) % LoadGraph::NUM_POINTS;
 
-	unsigned new_max;
+	guint64 new_max;
 	// both way, new_max is the greatest value
 	if (dmax >= g->net.max)
 		new_max = dmax;
@@ -386,7 +389,7 @@ net_scale (LoadGraph *g, unsigned din, unsigned dout)
 	// Round network maximum
 	//
 
-	const unsigned bak_max(new_max);
+	const guint64 bak_max(new_max);
 
 	if (ProcData::get_instance()->config.network_in_bits) {
 	  // TODO: fix logic to give a nice scale with bits
@@ -396,7 +399,7 @@ net_scale (LoadGraph *g, unsigned din, unsigned dout)
 	  new_max = 1.1 * new_max;
 	  // make sure max is not 0 to avoid / 0
 	  // default to 125 bytes == 1kbit
-	  new_max = std::max(new_max, 125U);
+	  new_max = std::max(new_max, 125UL);
 
 	} else {
 	  // round up to get some extra space
@@ -404,7 +407,7 @@ net_scale (LoadGraph *g, unsigned din, unsigned dout)
 	  new_max = 1.1 * new_max;
 	  // make sure max is not 0 to avoid / 0
 	  // default to 1 KiB
-	  new_max = std::max(new_max, 1024U);
+	  new_max = std::max(new_max, 1024UL);
 
 	  // decompose new_max = coef10 * 2**(base10 * 10)
 	  // where coef10 and base10 are integers and coef10 < 2**10
@@ -412,16 +415,16 @@ net_scale (LoadGraph *g, unsigned din, unsigned dout)
 	  // e.g: ceil(100.5 KiB) = 101 KiB = 101 * 2**(1 * 10)
 	  //      where base10 = 1, coef10 = 101, pow2 = 16
 
-	  unsigned pow2 = std::floor(log2(new_max));
-	  unsigned base10 = pow2 / 10;
-	  unsigned coef10 = std::ceil(new_max / double(1UL << (base10 * 10)));
+	  guint64 pow2 = std::floor(log2(new_max));
+	  guint64 base10 = pow2 / 10.0;
+	  guint64 coef10 = std::ceil(new_max / double(1UL <<(base10 * 10)));
 	  g_assert(new_max <= (coef10 * (1UL << (base10 * 10))));
 
 	  // then decompose coef10 = x * 10**factor10
 	  // where factor10 is integer and x < 10
 	  // so we new_max has only 1 significant digit
 
-	  unsigned factor10 = std::pow(10.0, std::floor(std::log10(coef10)));
+	  guint64 factor10 = std::pow(10.0, std::floor(std::log10(coef10)));
 	  coef10 = std::ceil(coef10 / double(factor10)) * factor10;
 
 	  // then make coef10 divisible by num_bars
@@ -429,12 +432,12 @@ net_scale (LoadGraph *g, unsigned din, unsigned dout)
 	    coef10 = coef10 + (g->num_bars() - coef10 % g->num_bars());
 	  g_assert(coef10 % g->num_bars() == 0);
 
-	  new_max = coef10 * (1UL << (base10 * 10));
-	  procman_debug("bak %u new_max %u pow2 %u coef10 %u", bak_max, new_max, pow2, coef10);
+	  new_max = coef10 * (1UL << guint64(base10 * 10));
+	  procman_debug("bak %lu new_max %lu pow2 %lu coef10 %lu", bak_max, new_max, pow2, coef10);
 	}
 
 	if (bak_max > new_max) {
-	  procman_debug("overflow detected: bak=%u new=%u", bak_max, new_max);
+	  procman_debug("overflow detected: bak=%lu new=%lu", bak_max, new_max);
 	  new_max = bak_max;
 	}
 
@@ -443,7 +446,7 @@ net_scale (LoadGraph *g, unsigned din, unsigned dout)
 	if ((0.8 * g->net.max) < new_max && new_max <= g->net.max)
 		return;
 
-	const float scale = 1.0f * g->net.max / new_max;
+	const double scale = 1.0f * g->net.max / new_max;
 
 	for (size_t i = 0; i < LoadGraph::NUM_POINTS; i++) {
 		if (g->data[i][0] >= 0.0f) {
@@ -452,7 +455,7 @@ net_scale (LoadGraph *g, unsigned din, unsigned dout)
 		}
 	}
 
-	procman_debug("rescale dmax = %u max = %u new_max = %u", dmax, g->net.max, new_max);
+	procman_debug("rescale dmax = %lu max = %lu new_max = %lu", dmax, g->net.max, new_max);
 
 	g->net.max = new_max;
 
@@ -468,7 +471,7 @@ get_net (LoadGraph *g)
 	guint32 i;
 	guint64 in = 0, out = 0;
 	GTimeVal time;
-	unsigned din, dout;
+	guint64 din, dout;
 
 	ifnames = glibtop_get_netlist(&netlist);
 
@@ -507,9 +510,9 @@ get_net (LoadGraph *g)
 	    g->net.time.tv_sec != 0) {
 		float dtime;
 		dtime = time.tv_sec - g->net.time.tv_sec +
-			(float) (time.tv_usec - g->net.time.tv_usec) / G_USEC_PER_SEC;
-		din   = static_cast<unsigned>((in  - g->net.last_in)  / dtime);
-		dout  = static_cast<unsigned>((out - g->net.last_out) / dtime);
+			(double) (time.tv_usec - g->net.time.tv_usec) / G_USEC_PER_SEC;
+		din   = static_cast<guint64>((in  - g->net.last_in)  / dtime);
+		dout  = static_cast<guint64>((out - g->net.last_out) / dtime);
 	} else {
 		/* Don't calc anything if new data is less than old (interface
 		   removed, counters reset, ...) or if it is the first time */
@@ -557,7 +560,7 @@ load_graph_update (gpointer user_data)
 	}
 
 	if (g->draw)
-		load_graph_draw (g);
+		load_graph_queue_draw (g);
 
 	g->render_counter++;
 
@@ -609,7 +612,6 @@ LoadGraph::LoadGraph(guint type)
     graph_buffer_offset(0),
     main_widget(NULL),
     disp(NULL),
-    gc(NULL),
     background(NULL),
     timer_index(0),
     draw(FALSE),
@@ -666,9 +668,9 @@ LoadGraph::LoadGraph(guint type)
 	case LOAD_GRAPH_MEM:
 		g->colors[0] = ProcData::get_instance()->config.mem_color;
 		g->colors[1] = ProcData::get_instance()->config.swap_color;
-		g->mem_color_picker = gsm_color_button_new (&g->colors[0], 
+		g->mem_color_picker = gsm_color_button_new (&g->colors[0],
 							    GSMCP_TYPE_PIE);
-		g->swap_color_picker = gsm_color_button_new (&g->colors[1], 
+		g->swap_color_picker = gsm_color_button_new (&g->colors[1],
 							     GSMCP_TYPE_PIE);
 		break;
 	case LOAD_GRAPH_NET:
@@ -687,8 +689,8 @@ LoadGraph::LoadGraph(guint type)
 
 	g->disp = gtk_drawing_area_new ();
 	gtk_widget_show (g->disp);
-	g_signal_connect (G_OBJECT (g->disp), "expose_event",
-			  G_CALLBACK (load_graph_expose), g);
+	g_signal_connect (G_OBJECT (g->disp), "draw",
+			  G_CALLBACK (load_graph_draw), g);
 	g_signal_connect (G_OBJECT(g->disp), "configure_event",
 			  G_CALLBACK (load_graph_configure), g);
 	g_signal_connect (G_OBJECT(g->disp), "destroy",
