@@ -525,11 +525,8 @@ procman_create_sysinfo_view(void)
   GtkWidget *distro_table;
 
   GtkWidget *hardware_table;
-  GtkWidget *memory_label;
-  GtkWidget *processor_label;
 
   GtkWidget *disk_space_table;
-  GtkWidget *disk_space_label;
 
   GtkWidget *header;
 
@@ -619,7 +616,7 @@ procman_create_sysinfo_view(void)
   g_free(markup);
 
   markup = procman::format_size(data->memory_bytes);
-  memory_label = add_row(GTK_TABLE(hardware_table), _("Memory:"),
+  add_row(GTK_TABLE(hardware_table), _("Memory:"),
                          markup, 0);
   g_free(markup);
 
@@ -634,7 +631,7 @@ procman_create_sysinfo_view(void)
       t = _("Processor:");
     }
 
-    processor_label = add_row(GTK_TABLE(hardware_table), t,
+    add_row(GTK_TABLE(hardware_table), t,
                               data->processors[i].c_str(), 1 + i);
 
     if(markup)
@@ -648,7 +645,7 @@ procman_create_sysinfo_view(void)
   g_free(markup);
 
   markup = procman::format_size(data->free_space_bytes);
-  disk_space_label = add_row(GTK_TABLE(disk_space_table),
+  add_row(GTK_TABLE(disk_space_table),
                              _("Available disk space:"), markup,
                              0);
   g_free(markup);
